/*
 *  Vazaar
 *  file: callbacks.c
 *
 *
 *  Copyright (C) 2005 Tomás Vírseda <kaskaras@yahoo.es>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gnome.h>
#include <libgnomevfs/gnome-vfs.h>
#include <glade/glade.h>
#include <libxml/xmlreader.h>
#include <math.h>

#include "support.h"
#include "vazaar.h"
#include "items.h"
#include "categories.h"
#include "callbacks.h"
#include "debug.h"

extern xmlDocPtr vazaar_doc;
extern xmlDocPtr items_doc;
extern xmlDocPtr cats_doc;
xmlNodePtr  node_found;

/**
 * on_mainWin_delete_event:
 * @widget:  widget que se pasa como parámetro con la señal
 * @app:  dato que se pasa como párametro con la señal
 *
 * Esta función pregunta al usuario si desea finalizar la ejecución del programa.
 *
 * En realidad el primer parámetro no hacen falta pero hay que ponerlo
 * por compatibilidad con el prototipado de la señal.
 *
 * return values: no devuelve nada.
 */
void
on_mainWin_delete_event(GtkWidget * nothing, Vazaar * app)
{	
    GtkWidget		*dialog;
    gchar			*title,
					*details;
    gint            result;
	
	g_return_if_fail(app != NULL);
	
    title = g_strdup(_("Are you sure?"));
	details = g_strdup (_("If your are editing an item or category and quit the program now, you can lose the data."));
    result = vazaar_gui_show_dialog (NULL, GTK_STOCK_DIALOG_QUESTION , title, details);
    switch (result) {
		case GTK_RESPONSE_OK:
			vazaar_gui_vazaar_destroy(app);	    
			break;
    }
}

/**
 * on_mainWin_configure_event:
 * @widget:  widget que se pasa como parámetro con la señal
 * @event:  evento que se pasa como parámetro con la señal
 * @app:  dato que se pasa como párametro con la señal
 *
 * Esta función pregunta al usuario si desea finalizar la ejecución del programa.
 * Reconfigura aspectos gráficos de la aplicación cuando se emite la señal
 * "configure-event" encargada de informar el widget se redimensiona o cambia de
 * posición.
 *
 * En realidad el primer parámetro no hacen falta pero hay que ponerlo
 * por compatibilidad con el prototipado de la señal.
 *
 * return values: no devuelve nada.
 */
void
on_mainWin_configure_event	(	GtkWidget * widget,
			   					GdkEventConfigure * event, 
								Vazaar * app)
{

    g_return_if_fail(app != NULL);

    gboolean        toggled = FALSE;
    guint           ancho = 0,
					alto = 0,
					hpancho = 0,
					vpalto = 0;
	
    toggled = gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON (app->viewcat));

    if (toggled) {
		gtk_window_get_size(GTK_WINDOW(app->window), &ancho, &alto);
		hpancho = rint(((ancho / 3.5) * (1.0)));
		vpalto = rint(((alto / 3.5) * (1.0)));
		gtk_paned_set_position(GTK_PANED(app->hpcat), hpancho);
		gtk_paned_set_position(GTK_PANED(app->vpitem), vpalto);
    }
}

void
on_add_category_activate		(	GtkMenuItem * menuitem, gpointer user_data){
	vazaar_gui_cat_add ((Vazaar *)user_data);
}

void
on_edit_category_activate		(	GtkMenuItem * menuitem, gpointer user_data){
	vazaar_gui_cat_edit ((Vazaar *)user_data);
}

void
on_delete_category_activate		(	GtkMenuItem * menuitem, gpointer user_data){
	vazaar_gui_cat_delete ((Vazaar *)user_data);
}

void
on_add_item_activate		(	GtkMenuItem * menuitem, gpointer user_data) {
	vazaar_gui_item_add ((Vazaar *)user_data);
}

void
on_edit_item_activate		(	GtkMenuItem * menuitem, gpointer user_data) {
	vazaar_gui_item_edit ((Vazaar *)user_data);
}

void
on_delete_item_activate		(	GtkMenuItem * menuitem, gpointer user_data) {
	vazaar_gui_item_del ((Vazaar *)user_data);
}

void
on_show_item_info_activate	(	GtkMenuItem * menuitem, gpointer user_data) {
	Vazaar *app = (Vazaar *)user_data;
	gboolean state;
	
	state = gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON (app->infoitem));
	
	gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (app->infoitem), !state);
	//on_tgbItemInfo_toggled (NULL, (Vazaar *)user_data);
}

void
on_show_categories_panel_activate	(	GtkMenuItem * menuitem, gpointer user_data) {
	Vazaar *app = (Vazaar *)user_data;
	gboolean state;
	
	state = gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON (app->viewcat));
	gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (app->viewcat), !state);
	//on_tgbViewCat_toggled (NULL, (Vazaar *)user_data);
}

void
on_quit_activate(GtkMenuItem * menuitem, gpointer user_data)
{

}

void
on_preferences_activate(GtkMenuItem * menuitem, gpointer user_data)
{

}





void
on_vazaar_activate(GtkMenuItem * menuitem, gpointer user_data)
{
}




/**
 * on_about_activate:
 * @menuitem:  elemento del menú que se pasa como parámetro con la señal
 * @app:  dato que se pasa como párametro con la señal
 *
 * Activa el diálogo "Acerca de..."
 *
 * En realidad el primer parámetro no hacen falta pero hay que ponerlo
 * por compatibilidad con el prototipado de la señal.
 *
 * return values: no devuelve nada.
 */
void
on_about_activate(GtkMenuItem * menuitem, Vazaar * app)
{
    const gchar	*authors[] 		= {	"Tomás Vírseda <kaskaras@yahoo.es>",	NULL};
    const gchar	*documenters[]	= {	"", 	NULL};
	const gchar	*artist[]		= {	"", 	NULL};
    gchar		*translators 	= 	"Czech: Zbynek Mrkvicka <zbynek@oachot.cz>\n" \
						"Spanish: Tomás Vírseda <kaskaras@yahoo.es>\n" \
						"Polish: Micha³ Kastelik <m.kastelik@osadkowski.com.pl>";
	gchar 		*licence_path	= NULL;
	gchar		*licence_doc	= NULL;
	gchar		*logo_path		= NULL;
	GdkPixbuf	*logo_pixbuf	= NULL;	
	gsize        content_len;
    GtkWidget	*aboutVazaar;
	GError        **error = NULL;
	
	licence_path = g_strdup_printf("%s", PACKAGE_DATA_DIR "/doc/" PACKAGE "/COPYING");
	g_file_get_contents	(	licence_path, 
							&licence_doc,
							&content_len, 
							error);
	if (error) {
		licence_doc = g_strdup_printf (_("Released under the GNU General Public License v2 or later"));
		error = NULL;
	}
	
	logo_path = g_strdup_printf("%s/%s", PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/system", "gnome-vazaar.svg");
	logo_pixbuf = gdk_pixbuf_new_from_file(logo_path, error);
	if (error) {
		g_message ("Error %d: %s", (*error)->code, (*error)->message);
		logo_pixbuf = NULL;
	}
	logo_pixbuf = gdk_pixbuf_add_alpha (logo_pixbuf, FALSE, 255, 255, 255);
	aboutVazaar = gtk_about_dialog_new ();
	gtk_about_dialog_set_logo (GTK_ABOUT_DIALOG(aboutVazaar), logo_pixbuf);
	gtk_about_dialog_set_name (GTK_ABOUT_DIALOG(aboutVazaar), "GNOME Vazaar");
	gtk_about_dialog_set_version (GTK_ABOUT_DIALOG(aboutVazaar), VERSION);
	gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG(aboutVazaar), _("Copyright(c) 2005 by Tomás Vírseda"));
	gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG(aboutVazaar), _("Organize your items"));
	gtk_about_dialog_set_license (GTK_ABOUT_DIALOG(aboutVazaar), licence_doc);
	gtk_about_dialog_set_website (GTK_ABOUT_DIALOG(aboutVazaar), "http://software-libre.org/projects/gnome-vazaar/");
	gtk_about_dialog_set_website_label (GTK_ABOUT_DIALOG(aboutVazaar), "software-libre.org");
	gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG(aboutVazaar), authors);
	gtk_about_dialog_set_documenters (GTK_ABOUT_DIALOG(aboutVazaar), documenters);
	gtk_about_dialog_set_translator_credits (GTK_ABOUT_DIALOG(aboutVazaar), translators);
	gtk_about_dialog_set_artists (GTK_ABOUT_DIALOG(aboutVazaar), artist);

    gtk_container_set_border_width(GTK_CONTAINER(aboutVazaar), 5);
    gtk_window_set_skip_taskbar_hint(GTK_WINDOW(aboutVazaar), TRUE);
    gtk_window_set_skip_pager_hint(GTK_WINDOW(aboutVazaar), TRUE);
    gtk_window_set_type_hint(GTK_WINDOW(aboutVazaar), GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_transient_for(GTK_WINDOW(aboutVazaar), GTK_WINDOW(app->window));
    gtk_widget_show(aboutVazaar);
}

/*
void
on_trvVazaarCat_row_activated	(	GtkWidget *nothing, 
									Vazaar * app)
{
    GtkTreeSelection *selection;
    GtkTreeModel   *model;
    GtkTreeIter     iter;
    gchar          *name;

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(app->treecat));
    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, CAT_COL_NOM, &name, -1);
		vazaar_gui_cat_edit(app);
		if (DEBUG)
			g_message(_("Selected category: %s"), name);
		g_free(name);
    }
}
*/

/**
 * on_mainWin_configure_event:
 * @widget:  widget que se pasa como parámetro con la señal
 * @app:  dato que se pasa como párametro con la señal
 *
 * Activa la categoria seleccionada por el usuario.
 *
 * En realidad el primer parámetro no hacen falta pero hay que ponerlo
 * por compatibilidad con el prototipado de la señal.
 *
 * return values: no devuelve nada.
 */
void
on_trvVazaarCat_cursor_changed	(	GtkWidget * nothing, Vazaar * app)
{
    GtkTreeSelection *selection;
    GtkTreeModel   *model;
    GtkTreeIter     iter;
    gchar          *name,
                   *desc,
					*syst,
                   *str;
    guint           nitems = 0;

    GtkWidget      *widget;
	xmlNodePtr root, child;
	
	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON (app->wastebasket), FALSE);
	
	app->citem = g_strdup_printf ("-");
	app->current_items = NULL;

	/*	Se vacía el contendor que contiene el actual widget */
    widget = gtk_paned_get_child2(GTK_PANED(app->vpitem));
    if (widget) {
		g_object_ref(app->vpitem);
		gtk_container_remove(GTK_CONTAINER(app->vpitem), widget);
    }
					
	/*	Si se ha seleccionado una categoría válida se ordenan los items */
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treecat));
    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_window_set_title (GTK_WINDOW(app->window), "GNOME Vazaar");	
		gtk_tree_model_get	(	model, &iter, 
								CAT_COL_NOM, &name, 
								CAT_COL_DES, &desc, 
								CAT_COL_SYS, &syst,
								-1);
		/* In oder to be HIG Compliant (a few more) we must put sensibility = FALSE
		thoses categories that we can delete. */
		root = xmlDocGetRootElement(cats_doc);
    	vazaar_xml_cat_find(root->xmlChildrenNode, g_strdup(name));
    	child = node_found;
		if ((child->children) || (vazaar_cmd_strings_compare (syst, "true") == 0)) {
			gtk_widget_set_sensitive (app->delcat, FALSE);
		}
		else {
			gtk_widget_set_sensitive (app->delcat, TRUE);
		}
		/**/
		
		nitems = vazaar_gui_items_load(app, name);
		str = g_strdup_printf(_("This category contains %d items"), nitems);
		gnome_appbar_set_status(GNOME_APPBAR (app->statusbar), str);
		app->current_items = g_list_sort (app->current_items, (GCompareFunc)vazaar_cmd_cat_sort_items); 
		
		/* Load First item in current category */
		GtkTreeSelection *item_selection = NULL;
		GList *first = g_list_first (app->current_items);
		if (first) {
			VazaarItemSort *item_sort = first->data;
			
			vazaar_gui_item_get_real(app, item_sort->item);
			item_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treeitem));
			gtk_tree_selection_select_iter (item_selection, &item_sort->iter);	
		}
		g_free(desc);
    }
}

void
on_trvVazaarItems_row_activated	(	GtkTreeView * treeview,
									GtkTreePath * path,
									GtkTreeViewColumn * column, 
									Vazaar * app)
{
	
}

/**
 * on_toggle_categorie:
 * @cell:	GtkCellRendererToggle que se pasa como parámetro con la señal
 * @path_str: GtkTRreePath convertido a gchar de la celda seleccionada
 * @app:  dato que se pasa como párametro con la señal
 *
 * Marca/Desmarca la categoria seleccionada por el usuario.
 *
 * En realidad el primer parámetro no hacen falta pero hay que ponerlo
 * por compatibilidad con el prototipado de la señal.
 *
 * return values: no devuelve nada.
 */
void
on_toggle_categorie	(	GtkCellRendererToggle * cell,
		    			gchar * path_str, 
						gpointer data)
{
    GtkTreeModel   *model = (GtkTreeModel *) data;
    GtkTreeIter     iter;
    GtkTreePath    *path = gtk_tree_path_new_from_string(path_str);
    gboolean        fixed;

    gtk_tree_model_get_iter(model, &iter, path);
    gtk_tree_model_get(model, &iter, CAT_COL_CHK, &fixed, -1);
    fixed ^= 1;
    gtk_tree_store_set(	GTK_TREE_STORE(model), &iter, 
						CAT_COL_CHK, fixed,
		       			-1);
    gtk_tree_path_free(path);
}

/**
 * on_toggle_categorie:
 * @cell:	GtkCellRendererToggle que se pasa como parámetro con la señal
 * @path_str: GtkTRreePath convertido a gchar de la celda seleccionada
 * @app:  dato que se pasa como párametro con la señal
 *
 * Marca/Desmarca la categoria seleccionada por el usuario.
 *
 * En realidad el primer parámetro no hacen falta pero hay que ponerlo
 * por compatibilidad con el prototipado de la señal.
 *
 * return values: no devuelve nada.
 */




void
on_popup_add_category_activate()
{
}


void
on_popup_delete_category_activate()
{
}


void
on_pixmapentry_add_cat_activate()
{
}


void
on_popup_edit_category_activate()
{
}






void
on_trvVazaarItems_cursor_changed()
{
}


/*
 * Categories callbacks 
 */
void
on_tlbAddCat_clicked(GtkWidget * nothing, Vazaar * app)
{
    vazaar_gui_cat_add(app);
}

void
on_tlbEditCat_clicked(GtkWidget * nothing, Vazaar * app)
{
    vazaar_gui_cat_edit(app);
}

void
on_tlbDelCat_clicked(GtkWidget * nothing, Vazaar * app)
{
    vazaar_gui_cat_delete(app);
}

void
on_tlbExpCat_clicked(GtkWidget * nothing, Vazaar * app)
{
    vazaar_gui_expand_categories (app);
}

void
on_tlbColCat_clicked(GtkWidget * nothing, Vazaar * app)
{
    vazaar_gui_collapse_categories (app);
}

/**
 * on_tgbViewCat_toggled:
 * @widget:  widget que se pasa como parámetro con la señal
 * @app:  dato que se pasa como párametro con la señal
 *
 * Show/Hide categories panel.
 *
 * En realidad el primer parámetro no hacen falta pero hay que ponerlo
 * por compatibilidad con el prototipado de la señal.
 *
 * return values: no devuelve nada.
 */
void
on_tgbViewCat_toggled(GtkWidget * widget, Vazaar * app)
{
    g_return_if_fail(app->viewcat != NULL);

    gboolean        toggled = FALSE;
    guint           ancho = 0,
	alto = 0,
	hpancho = 0,
	vpalto = 0;

    toggled = gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON (app->viewcat));
    if (toggled) {
		gtk_window_get_size(GTK_WINDOW(app->window), &ancho, &alto);
		hpancho = rint(((ancho / 3.5) * (1.0)));
		vpalto = rint(((alto / 3.5) * (1.0)));
		gtk_paned_set_position(GTK_PANED(app->hpcat), hpancho);
		gtk_paned_set_position(GTK_PANED(app->vpitem), vpalto);
		gtk_widget_show (app->frmitems);
    } else {
		gtk_paned_set_position(GTK_PANED(app->hpcat), 0);
		gtk_paned_set_position(GTK_PANED(app->vpitem), 0);
    }
}


/*
 * Items callbacks 
 */
void
on_tlbItemAdd_clicked(GtkWidget * nothing, Vazaar * app)
{
    vazaar_gui_item_add(app);
}

void
on_tlbItemDel_clicked(GtkWidget * nothing, Vazaar * app)
{
    vazaar_gui_item_del(app);
}

void
on_tlbItemEdit_clicked(GtkWidget * nothing, Vazaar * app)
{
    vazaar_gui_item_edit(app);
}



void
on_tgbItemInfo_toggled (GtkWidget *nothing, Vazaar *app) {
    gboolean        toggled = FALSE;

    toggled = gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(app->infoitem));
	if (toggled) {
		gtk_widget_show (app->frameInfo);
	}
	else {
		gtk_widget_hide (app->frameInfo);
	}
}

void
vazaar_gui_treeitem_row_changed	(	GtkWidget * nothing, Vazaar * app)
{
    vazaar_gui_item_get(app);
}

void 
on_tlbItemFirst_clicked			(	GtkWidget *nothing, Vazaar	*app) {
	GList	*first = NULL;
	
	first = g_list_first (app->current_items);

	if (first) {
		VazaarItemSort *item_sort_first = first->data;
		GtkTreeSelection	*selection;
		
		/* No refresh the same item if selected */
		if (g_utf8_collate (item_sort_first->item->id, app->citem) != 0) {
			selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treeitem));
			gtk_tree_selection_select_iter (selection, &item_sort_first->iter);		
			vazaar_gui_item_get_real (app, item_sort_first->item);
		}
	}
}

void 
on_tlbItemPrevious_clicked		(	GtkWidget *nothing, Vazaar	*app) {
	GList	*aux = NULL,
			*previous = NULL;
	
	aux = app->current_items;
	while (aux) {
		VazaarItemSort *item_sort = aux->data;
		if (g_utf8_collate (item_sort->item->id, app->citem) == 0) {
			previous = g_list_previous (aux);
		}		
		aux = aux->next;
	}

	if (previous) {
		VazaarItemSort *item_sort_previous = previous->data;
		GtkTreeSelection	*selection;
		
		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treeitem));
		gtk_tree_selection_select_iter (selection, &item_sort_previous->iter);		
		vazaar_gui_item_get_real (app, item_sort_previous->item);
	}
}

void on_tlbItemRefresh_clicked	(	GtkWidget *nothing, Vazaar	*app) {
	app->force_reload = TRUE;
	vazaar_gui_item_get (app);
	app->force_reload = FALSE;
}

void on_tlbItemNext_clicked		(	GtkWidget *nothing, Vazaar	*app) {
	GList	*aux = NULL,
			*next = NULL;
	
	aux = app->current_items;
	while (aux) {
		VazaarItemSort *item_sort = aux->data;
		if (g_utf8_collate (item_sort->item->id, app->citem) == 0) {
			next = g_list_next (aux);
		}		
		aux = aux->next;
	}

	if (next) {
		VazaarItemSort *item_sort_next = next->data;
		GtkTreeSelection	*selection;
		
		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treeitem));
		gtk_tree_selection_select_iter (selection, &item_sort_next->iter);		
		vazaar_gui_item_get_real (app, item_sort_next->item);
	}
}

void on_tlbItemLast_clicked		(	GtkWidget *nothing, Vazaar	*app) {
	GList	*last = NULL;
	
	last = g_list_last (app->current_items);

	if (last) {
		VazaarItemSort *item_sort_last = last->data;
		GtkTreeSelection	*selection;
		
		/* No refresh the same item if selected */
		if (g_utf8_collate (item_sort_last->item->id, app->citem) != 0) {
			selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treeitem));
			gtk_tree_selection_select_iter (selection, &item_sort_last->iter);		
			vazaar_gui_item_get_real (app, item_sort_last->item);
		}
	}
}
